from http.client import HTTPException

import socketio
from fastapi import FastAPI, APIRouter
from fastapi.exceptions import RequestValidationError
from starlette.exceptions import HTTPException
from starlette.responses import PlainTextResponse

from server.model.data import Game
# Game state
from server.model.players import RandomPlayer
from server.ws import sio

game = Game()

# Server
app = FastAPI()
router = APIRouter()
#
# app.add_middleware(
#     CORSMiddleware,
#     allow_origins=["*"],
#     allow_credentials=True,
#     allow_methods=["*"],
#     allow_headers=["*"],
# )

# Create and mount SocketIO app
sio_asgi_app = socketio.ASGIApp(
    socketio_server=sio,
    other_asgi_app=app
)
app.add_route("/socket.io/", route=sio_asgi_app, methods=['GET', 'POST'])
app.add_websocket_route("/socket.io/", route=sio_asgi_app)


@router.get("/")
async def hello_world():
    return "Hello, gentle[wo]man"


@router.post("/join")
async def join(player_name: str):
    if game.add_player(RandomPlayer(player_name)  # TODO: Let user play
                       ):
        return "Welcome %s, %i players currently waiting!" % (player_name, len(game.players))
    else:
        raise HTTPException(status_code=403, detail=f"{player_name} already connected, choose another name.")


app.include_router(router)


@app.exception_handler(HTTPException)
async def http_exception_handler(request, exc):
    return PlainTextResponse(str(exc.detail), status_code=exc.status_code)


@app.exception_handler(RequestValidationError)
async def validation_exception_handler(request, exc):
    return PlainTextResponse(str(exc), status_code=400)


if __name__ == '__main__':
    import uvicorn

    uvicorn.run("app:app", host="0.0.0.0", port=9042, reload=True)
